#ifndef BYKMATH_H
#define BYKMATH_H

#include <math.h>
#include <memory.h>

//Math classes
class POINT2; class POINT3; class POINT4;
class RECT2; class RECT3; class RECT4;
class PLANE;
class QUATERNION;
class MATRIX2X2; class MATRIX3X3; class MATRIX4X4; class MATRIXNXM;

// POINT2
class POINT2
{
public:
	POINT2(); //constructor dft
	POINT2(float tx,float ty);

	//Variables
	float x,y;

	//Funciones
	const POINT2 operator=  (const POINT2 &p);
	const POINT2 operator+  (const POINT2 &p);
	const POINT2 operator+  (void);
	const POINT2 operator+= (const POINT2 &p);
	const POINT2 operator-  (const POINT2 &p);
	const POINT2 operator-  (void);
	const POINT2 operator-= (const POINT2 &p);
	const POINT2 operator*= (const float &s);
	const POINT2 operator/= (const float &s);
	const POINT2 operator * (const float &s);
	const POINT2 operator * (const POINT2 &p);
	const POINT2 operator/  (const float s);
	const bool	 operator== (const POINT2 &p);
	const bool   operator!= (const POINT2 &p);
	const float& operator[] (int n);
	const float Dot(POINT2 &p);
	const float Length(void);
	const float SquaredDistance(POINT2 &p);
	const float Distancia(float px,float py);
	const float Distancia(const POINT2 &p);
	const POINT2 Unit(void);
	const POINT2 Transladar(float tx,float ty);
	const POINT2 RotarZ(float az);
	const POINT2 RotarZAPos(float az,float px,float py);
	const POINT2 Scalar(float scx,float scy);
	const POINT2 Lerp(POINT2 &p,float t);
	const void Normalize(void);
	const void SetProperties(float sx,float sy);

	~POINT2(); //deconstructor
};

// POINT3
class POINT3
{
public:
	POINT3(); //constructor dft
	POINT3(float tx,float ty,float tz);

	//Variables
	float x,y,z;

	//Funciones
	const POINT3 operator=  (const POINT3 &p);
	const POINT3 operator+  (const POINT3 &p);
	const POINT3 operator+  (void);
	const POINT3 operator+= (const POINT3 &p);
	const POINT3 operator-  (const POINT3 &p);
	const POINT3 operator-  (void);
	const POINT3 operator-= (const POINT3 &p);
	const POINT3 operator*= (const float &s);
	const POINT3 operator/= (const float &s);
	const POINT3 operator * (const float &s);
	const POINT3 operator * (const POINT3 &p);
	const POINT3 operator/  (const float s);
	const bool	 operator== (const POINT3 &p);
	const bool   operator!= (const POINT3 &p);
	const float& operator[] (int n);
	const float Dot(POINT3 &p);
	const float Length(void);
	const float SquaredDistance(POINT3 &p);
	const float Distancia(float px,float py,float pz);
	const float Distancia(const POINT3 &p);
	const POINT3 Cross(POINT3 &p);
	const POINT3 Unit(void);
	const POINT3 Transladar(float tx,float ty,float tz);
	const POINT3 RotarX(float ax);
	const POINT3 RotarY(float ay);
	const POINT3 RotarZ(float az);
	const POINT3 RotarXAPos(float ax,float px,float py,float pz);
	const POINT3 RotarYAPos(float ay,float px,float py,float pz);
	const POINT3 RotarZAPos(float az,float px,float py,float pz);
	const POINT3 Scalar(float scx,float scy,float scz);
	const POINT3 Lerp(POINT3 &p,float t);
	const void Normalize(void);
	const void SetProperties(float sx,float sy,float sz);

	~POINT3(); //deconstructor
};

// POINT4
class POINT4
{
public:
	POINT4(); //constructor dft
	POINT4(float tx,float ty,float tz,float tw);

	//Variables
	float x,y,z,w;

	//Funciones
	const POINT4 operator=  (const POINT4 &p);
	const POINT4 operator+  (const POINT4 &p);
	const POINT4 operator+  (void);
	const POINT4 operator+= (const POINT4 &p);
	const POINT4 operator-  (const POINT4 &p);
	const POINT4 operator-  (void);
	const POINT4 operator-= (const POINT4 &p);
	const POINT4 operator*= (const float &s);
	const POINT4 operator/= (const float &s);
	const POINT4 operator * (const float &s);
	const POINT4 operator * (const POINT4 &p);
	const POINT4 operator/  (const float s);
	const bool	 operator== (const POINT4 &p);
	const bool   operator!= (const POINT4 &p);
	const float& operator[] (int n);

	const float Dot(POINT4 &p);
	const float Length(void);
	const float SquaredDistance(POINT4 &p);
	const float Distancia(float px,float py,float pz,float pw);
	const float Distancia(const POINT4 &p);
	const POINT4 Cross(POINT4 &p);
	const POINT4 Unit(void);
	const POINT4 Transladar(float tx,float ty,float tz,float tw);
	const POINT4 RotarX(float ax);
	const POINT4 RotarY(float ay);
	const POINT4 RotarZ(float az);
	const POINT4 RotarXAPos(float ax,float px,float py,float pz);
	const POINT4 RotarYAPos(float ay,float px,float py,float pz);
	const POINT4 RotarZAPos(float az,float px,float py,float pz);
	const POINT4 Scalar(float scx,float scy,float scz,float scw);
	const POINT4 Lerp(POINT4 &p,float t);
	const void Normalize(void);
	const void SetProperties(float sx,float sy,float sz,float sw);

	~POINT4(); //deconstructor
};

// RECT2
class RECT2
{
public:
	RECT2(); //constructor dft
	RECT2(float x1,float y1,float x2,float y2);
	RECT2(POINT2 p1,POINT2 p2);
	RECT2(float tm,float te);
	
	//Variables
	float m,e;

	//Funciones
	const RECT2 operator=  (const RECT2 &r);
	const bool	operator== (const RECT2 &r);
	const bool  operator!= (const RECT2 &r);
	const bool  paralelas  (const RECT2 &r);
	const bool  perpendiculares (const RECT2 &r);
	const POINT2 intersectar (const RECT2 &r);
	const POINT2 iejes     (void);
	const float calcularXY (const float Y);
	const float calcularYX (const float X);
	const float distancia  (const RECT2 &r);
	const float distancia  (const POINT2 &p);
	const void SetProperties(float sm,float se);

	~RECT2();//deconstructor
};

// RECT3
class RECT3
{
public:
	RECT3(); //constructor dft
	RECT3(float x1,float y1,float z1,float x2,float y2,float z2);
	RECT3(POINT3 p1,POINT3 p2);
	RECT3(float tmy,float tey,float tmz,float tez);

	//Variables
	float my,ey,mz,ez;

	//Funciones
	const RECT3 operator=  (const RECT3 &r);
	const bool	operator== (const RECT3 &r);
	const bool  operator!= (const RECT3 &r);
	const bool  paralelas  (const RECT3 &r);
	const bool  perpendiculares (const RECT3 &r);
	const POINT3 intersectar (const RECT3 &r);
	const POINT3 iejes     (void);
	const float calcularXY (const float Y);
	const float calcularXZ (const float Z);
	const float calcularYX (const float X);
	const float calcularYZ (const float Z);
	const float calcularZX (const float X);
	const float calcularZY (const float Y);
	const float distancia  (const RECT3 &r);
	const float distancia  (const POINT3 &p);
	const void SetProperties(float smy,float sey,float smz,float sez);
	~RECT3(); //deconstructor
};

// PLANE
class PLANE
{
public:
	PLANE(); //constructor dft
	PLANE(POINT3 n,float td);
	PLANE(POINT3 p,POINT3 q,POINT3 r);
	PLANE(float ta,float tb,float tc,float td);

	//Variables
	float a,b,c,d;

	//Funciones
	const PLANE operator=  (const PLANE &r);
	const bool	operator== (const PLANE &r);
	const bool  operator!= (const PLANE &r);
	const POINT3 intersectRect(RECT3 &r);
	const char	intersectTriangle (const POINT3 &p1,const POINT3 &p2,const POINT3 &p3);
	const float& operator[](int n);
	const void SetProperties(POINT3 sn,float sd);
	const void SetProperties(POINT3 sp,POINT3 sq,POINT3 sr);
	const void SetProperties(float sa,float sb,float sc,float sd);

	~PLANE(); //deconstructor
};

// QUATERNION
class QUATERNION
{
public:
	QUATERNION(); //constructor dft
	QUATERNION(float tx,float ty,float tz,float tw);

	//Variables
	float x,y,z,w;

	//Funciones
	const QUATERNION operator=  (const QUATERNION &qt);
	const QUATERNION operator+  (const QUATERNION &qt);
	const QUATERNION operator+  (void);
	const QUATERNION operator+= (const QUATERNION &qt);
	const QUATERNION operator-  (const QUATERNION &qt);
	const QUATERNION operator-  (void);
	const QUATERNION operator-= (const QUATERNION &qt);
	const QUATERNION operator*  (const QUATERNION &qt);
	const QUATERNION operator*  (const float s);
	const QUATERNION operator*= (const QUATERNION &qt);
	const QUATERNION operator*= (const float s);
	const QUATERNION operator/  (const float s);
	const QUATERNION operator/= (const float s);
	const bool		 operator== (const QUATERNION &qt);
	const bool		 operator!= (const QUATERNION &qt);
	const float&	 operator[] (int n);
	const float Dot(QUATERNION &qt);
	const float Length(void);
	const QUATERNION Normal(void);
	const void CrearMatrix(MATRIX3X3 &mat);
	const void CrearMatrix(MATRIX4X4 &mat);
	const void CrearFromMatrix(MATRIX3X3 &mat);
	const void CrearFromMatrix(MATRIX4X4 &mat);
	const QUATERNION Slerp(QUATERNION &q1,QUATERNION &q2,float t);
	const QUATERNION Lerp(QUATERNION &q1,QUATERNION &q2,float t);
	const void Normalize(void);
	const void SetProperties(float sx,float sy,float sz,float sw);

	~QUATERNION(); //desconstructor
};

// MATRIX2X2
class MATRIX2X2
{
public:
	MATRIX2X2(); //constructor dft
	MATRIX2X2(float _a,float _b,float _c,float _d);

	//Variables
	float m[2][2];

	//Funciones
	const MATRIX2X2 operator=  (const MATRIX2X2 &mt);
	const MATRIX2X2 operator+  (const MATRIX2X2 &mt);
	const MATRIX2X2 operator+  (void);
	const MATRIX2X2 operator+= (const MATRIX2X2 &mt);
	const MATRIX2X2 operator-  (const MATRIX2X2 &mt);
	const MATRIX2X2 operator-  (void);
	const MATRIX2X2 operator-= (const MATRIX2X2 &mt);
	const MATRIX2X2 operator*  (const MATRIX2X2 &mt);
	const MATRIX2X2 operator*  (const float s);
	const MATRIX2X2 operator*= (const MATRIX2X2 &mt);
	const MATRIX2X2 operator*= (const float s);
	const MATRIX2X2 operator/  (const MATRIX2X2 &mt);
	const MATRIX2X2 operator/  (const float s);
	const MATRIX2X2 operator/= (const MATRIX2X2 &mt);
	const MATRIX2X2 operator/= (const float s);
	const bool		operator== (const MATRIX2X2 &mt);
	const bool		operator!= (const MATRIX2X2 &mt);
//	const float&	operator[][] (int n,int o);
	const MATRIX2X2 Inverse(void);
	const MATRIX2X2	Transpose(void);
	const void SetProperties(float _a,float _b,float _c,float _d);

	~MATRIX2X2(); //deconstructor
};

// MATRIX3X3
class MATRIX3X3
{
public:
	MATRIX3X3(); //constructor dft
	MATRIX3X3(float _a,float _b,float _c,float _d,float _e,float _f,float _g,float _h,float _i);

	//Variables
	float m[3][3];

	//Funciones
	const MATRIX3X3 operator=  (const MATRIX3X3 &mt);
	const MATRIX3X3 operator+  (const MATRIX3X3 &mt);
	const MATRIX3X3 operator+  (void);
	const MATRIX3X3 operator+= (const MATRIX3X3 &mt);
	const MATRIX3X3 operator-  (const MATRIX3X3 &mt);
	const MATRIX3X3 operator-  (void);
	const MATRIX3X3 operator-= (const MATRIX3X3 &mt);
	const MATRIX3X3 operator*  (const MATRIX3X3 &mt);
	const MATRIX3X3 operator*  (const float s);
	const MATRIX3X3 operator*= (const MATRIX3X3 &mt);
	const MATRIX3X3 operator*= (const float s);
	const MATRIX3X3 operator/  (const MATRIX3X3 &mt);
	const MATRIX3X3 operator/  (const float s);
	const MATRIX3X3 operator/= (const MATRIX3X3 &mt);
	const MATRIX3X3 operator/= (const float s);
	const bool		operator== (const MATRIX3X3 &mt);
	const bool		operator!= (const MATRIX3X3 &mt);
//	const float&	operator[][] (int n,int o);
	const MATRIX3X3 Inverse(void);
	const MATRIX3X3	Transpose(void);
	const void SetProperties(float _a,float _b,float _c,float _d,float _e,float _f,float _g,float _h,float _i);

	~MATRIX3X3(); //deconstructor
};

// MATRIX4X4
class MATRIX4X4
{
public:
	MATRIX4X4(); //constructor dft
	MATRIX4X4(float _a,float _b,float _c,float _d,float _e,float _f,float _g,float _h,float _i,float _j,float _k,float _l,float _m,float _n,float _o,float _p);

	//Variables
	float m[4][4];

	//Funciones
	const MATRIX4X4 operator=  (const MATRIX4X4 &mt);
	const MATRIX4X4 operator+  (const MATRIX4X4 &mt);
	const MATRIX4X4 operator+  (void);
	const MATRIX4X4 operator+= (const MATRIX4X4 &mt);
	const MATRIX4X4 operator-  (const MATRIX4X4 &mt);
	const MATRIX4X4 operator-  (void);
	const MATRIX4X4 operator-= (const MATRIX4X4 &mt);
	const MATRIX4X4 operator*  (const MATRIX4X4 &mt);
	const MATRIX4X4 operator*  (const float s);
	const MATRIX4X4 operator*= (const MATRIX4X4 &mt);
	const MATRIX4X4 operator*= (const float s);
	const MATRIX4X4 operator/  (const MATRIX4X4 &mt);
	const MATRIX4X4 operator/  (const float s);
	const MATRIX4X4 operator/= (const MATRIX4X4 &mt);
	const MATRIX4X4 operator/= (const float s);
	const bool		operator== (const MATRIX4X4 &mt);
	const bool		operator!= (const MATRIX4X4 &mt);
//	const float&	operator[][] (int n,int o);
	const float		CoFactor(char row,char col);
	const float		Determinant(void);
	const MATRIX4X4 CFInverse(void);
	const MATRIX4X4 Inverse(void);
	const MATRIX4X4 PRInverse(void);
	const MATRIX4X4	Transpose(void);
	const POINT3	Point(const POINT3 &p);
	const POINT4	Point(const POINT4 &p);
	const void SetProperties(float _a,float _b,float _c,float _d,float _e,float _f,float _g,float _h,float _i,float _j,float _k,float _l,float _m,float _n,float _o,float _p);

	~MATRIX4X4(); //deconstructor
};

	#ifndef M_PI
		#define M_PI	(float)3.1415926535898
	#endif /* M_PI */

	#define byKabs(x)			(float)(fabs(x))
	#define byKcos(a)			(float)(cosf(a*M_PI/180.0f))
	#define byKsin(a)			(float)(sinf(a*M_PI/180.0f))
	#define byKtan(a)			(float)(tanf(a*M_PI/180.0f))
	#define byKacos(a)			(float)(acosf(a)/M_PI*180.0f)
	#define byKasin(a)			(float)(asinf(a)/M_PI*180.0f)
	#define byKatan(a)			(float)(atanf(a)/M_PI*180.0f)
	#define byKmod(a,b)			(float)(fmodf(a,b))
	#define byKpotencia(a,b)	(float)(powf(a,b))
	#define byKraiz(a,b)		(float)(powf(a,1.0f/b))
	#define byKlog(a)			(float)(log10f(a))
	#define byKln(a)			(float)(logf(a))
	#define byKlg(a,b)			(float)(log10f(a)/log10f(b))
	#define byKex(a)			(float)(expf(a))
	#define byK10x(a)			(float)(powf(10.0f,a))
	#define byKexp(a,b)			(float)(a*powf(10.0f,b))

#endif


