/* FILE: model.h
 *		Read shape files and render using AllegroGL
 *
 * C++ version
 *
 * Monstrous Software (c) 2001
 */


#ifndef __modelosMS3D_h__
#define __modelosMS3D_h__

#include <iostream>
#include <string>
#include <fstream>
#include <vector>

using namespace std;

#include <stdio.h>
#include <math.h>
#include <string.h>

#include "texturas.h"
#include "zeru_mat.h"
#include "constantes.h"

#include "rendereable.h"


/* MilkShape constants */
#define MS_MAX_NAME             32
#define MS_MAX_PATH             256


// -----------

class Vec
{
 public:
  void	transform( const Matrix& m );
  void	transform3( const Matrix& m );
  float	x,y,z;
  float	w;	// 4th homogenous coordinate 
  float	u,v;	// texture position
  int bone;	// index into Model bones array
};



class Tri
{
 public:
  int v[3];	// indices into vertices[]
  int n[3];	// indices into normals[], normal per vertex
};



class KeyFrame
{
 public:
  float	Time;
  float	Value[3];
};


class Bone
{
 public:
  Bone();
  ~Bone();
  bool	loadFromMs3dAsciiSegment( FILE *file );
  void	fixup( void );
  void	advanceTo( float time );
  void	initialize( void );
  //	private:
  char	Name[MS_MAX_NAME];
  char	ParentName[MS_MAX_NAME];
  Bone	*Parent;						// pointer to parent bone (or NULL)
  float	startPosition[3];
  float	startRotation[3];
  Matrix	m_relative;				// fixed transformation matrix relative to parent 
  Matrix	m_final;				// absolute in accordance to animation
  int		NumPositionKeys;
  KeyFrame *PositionKeyFrames;
  int		NumRotationKeys;
  KeyFrame *RotationKeyFrames;
};




class Shape 
{
 public:
  /*	Constructor. */
  Shape();
  
  /*	Destructor. */
  ~Shape();

  bool  loadFromMs3dAsciiSegment( FILE *file , float Escala);
  bool  dibujame(Bone *bones, float Escala);

  float getRadio(){return Radio;};
  float getAlto(){return Alto;};
  void GetMaxMin(float *MaxMin, Bone *bones, float Escala);
  int	num_vertices;
  Vec	*vertices;
  int	num_triangles;
  Tri	*triangles;
  int	num_normals;
  z3DVector	*normals;

  bool *MayaIA;

 private:
  float Radio; 
  float Alto; 
};



class Model: public rendereable
{
 public:
  Model();
  ~Model();

 static bool Mallado;
  
  bool Inicia(string Modelo, string Textura, string Animacion, float Escala);
  bool Inicia(string Modelo, string Textura, string Arma01, string TexArma01, string Animacion, float Escala);
  void dibujame(float x, float y, float z, float frame, float angulo, float armaAngulo);
  void dibujame( float x, float y, float z, float frame, float angulo, z3DVector Normal, float armaAngulo);

  void  dibujameSelect(float x, float y, float z, float frame, float angulo);
  bool	linkBones( void );
  void	initializeBones( void );
  void	attachSkin(Shape &mod);
  int getNumFrames (){return (int)MaxTime;};
  inline float getRadio() { return (modBicho.getRadio() * 0.004 * Escala); };
  inline float getAlto() { return (modBicho.getAlto());};

  inline bool* getIAMaya() {return modBicho.MayaIA; };

  Animacion getFrames(string anim);

 private:

  int NumAnimaciones;

  string ModeloNombre;
  string ArmaNombre;
  string AnimacionNombre;

  float MaxMin[6]; // Maximos  y Minimos para dibujar la caja de Seleccion.
  float	MaxTime;	// length of animation (= Frames)
  float Escala;
  Textura texBicho;
  Textura texArma;
  Shape	modBicho;
  Shape modArma;
  int		*material_indices;
  int		num_bones;
  int numArmas;
  Bone	*bones;


  Animacion * Animaciones;
  bool loadAnimaciones(FILE *file);

  
};


#endif



