/*=[[ Solar OpenSource Org (c) 2003 ]]=*/

//***************************************************************
//*  [[ A D V E R T E N C I A ]] =[ http://Solar-OpenSource.Org ]=
//*
//*  LICENCIA PARA LA DISTRIBUCIÓN DE WAR 3D
//*  1.- WAR 3D es un programa que podrá ser distribuido bajo los términos de la Licencia Pública General, además deberán cumplirse las siguientes condiciones.
//*  2.- Al copiar cualquier parte del código para ponerlo en un programa diferente a WAR 3D, el autor de dicho programa estará de acuerdo en que el código completo de su programa pasa a ser software libre, por lo deberá estar apegado a los términos establecidos por la licencia LGP. Es decir que, el código de WAR 3D solo puede ser tomado si se va a usar en otro proyecto de software libre. De lo contrario absténgase de usar el código o parte de el código de WAR3D.
//*  3.- Queda estrictamente prohibido modificar los nombres que aparecen en los créditos del programa, solo el grupo de desarrollo de  Solar-OpenSource podrá añadir nuevos nombres en los créditos. Si ha hecho alguna modificación y quiere que su nombre aparezca en los créditos debe ponerse en contacto con nosotros.
//*  4.- Cualquier modificación del código debe apegarse en lo posible a las normas que ha establecido el grupo de desarrollo de Solar-OpenSource, esto es para mayor claridad en el código. Dichas normas pueden encontrarse en nuestra pagina web http://www.solar-opensource.org
//*  5.- Puede copiar y distribuir el programa siempre y cuando no olvide: Acompañarlo con el código fuente completo correspondiente, legible para la máquina, así como el código escrito por usted o las modificaciones hechas al código original.
//*  6.- No debe distribuirse el ejecutable sin el código fuente.
//*  7.- Este texto no debe suprimirse. Las distribuciones deben acompañarse con la información que se recibió así como el ofrecimiento para distribuir el código fuente correspondiente.
//*
//***************************************************************

#ifndef __oObjetos_h__
#define __oObjetos_h__

#include <iostream>
#include <string>
#include <fstream>

using namespace std;

#include "constantes.h"
#include <math.h>
#include "zeru_mat.h"


#include "escribe.h"
#include "SoundServer.h"
#include "animaciones.h"
#include "ServidorDeDisparos.h"
#include "ServidorDeSprites.h"
#include "army.h"


#ifndef SolarSocketsDebug
#include <SolarSockets/SolarSockets.h> // Libreria http://solar-opensource.org
#else
#include "../SolarSockets/include/SolarSockets.h" // Libreria http://solar-opensource.org
#endif

#define EstadoQuieto            -100
#define EstadoDisparando        -200
#define EstadoMuertoDestruido   -300 


#define MinimoDesaparece 0.25

class oObjetos
{

 private:

  z2DVector localizacion;
  bool Mio;

 protected:

  // -- Sirve para llevar las animaciones al azar. --
  float ContadorAzar;

  int Modo;

  int Estado;
  int EstadoAnterior;

  float Retardo;
  float Anima;
  int Sangre;
  float Angulo;
  float AnguloDeDisparo;
  float Transparencia;

  float DesaparecerMuerto;
  
  UnidadArmyBase * MyArmy;
  animaciones *AnimPlayer;

  ServidorDeSprites Explosion;

  bool ReproduciendoSonido;

  int PresaOSeguirID;
  float DisparoRetardo;

  float Azar;

 public:

  int ID;
  int internetID;

  bool Seleccionado;
  float Altura;


  // --------------------------
  // Constructores y destructor
  // --------------------------
 public:
  oObjetos (float x, float y, int ID, int internetID, bool Mio);
  virtual ~oObjetos(){};


  // --------------------------
  // Render
  // --------------------------
 public:
  virtual void dibujame(float Altura, z3DVector N, bool Seleccion) = 0; 
  virtual float getAngulo();   


  // --------------------------
  // Amigos enemigos 
  // --------------------------
 public:
  inline bool getMio() {return Mio;};


  // --------------------------
  // Localizacion 
  // --------------------------
 public:
  inline z2DVector getLocalizacion (){return localizacion;};
  inline void setLocalizacion (float x, float y) {localizacion.x = x; localizacion.y = y;};


  // --------------------------
  // Army
  // --------------------------
 public:
  void setArmy(UnidadArmyBase *MyArmy);
  virtual void onSetArmy() = 0;

  inline float getCosto() {return MyArmy->costo;};
  inline int getSangre() {return Sangre;};
  inline int getSangreTotal() {return MyArmy->sangre;};
  inline rendereable *getModelo() {return MyArmy->Modelo;};
  inline float getRadio() {return MyArmy->Modelo->getRadio();};
  inline int getTipo() {return MyArmy->ID;};
  inline float getAlcance() {return MyArmy->alcance;};
  inline int getPoder() {return MyArmy->danio;};

  // --------------------------
  // Modo y estado
  // --------------------------
 public:
  inline int getEstado(){return Estado;};
  inline int getModo(){return Modo;};
 public:
  virtual void setEstado(int Estado){this->Estado = Estado;};
  virtual void setModo (int Modo, float x, float y, int Extra, float NetX, float NetY){this->Modo = Modo;};


  // --------------------------
  // Animaciones
  // --------------------------
 public:
  inline float getAnima() {return Anima;};
  virtual void playAnimacion () = 0;

  // -- EVENTOS --

  // --------------------------
  // Tiempo y retardos
  // --------------------------
 public:
  void setRetardo(float Retardo);
 protected:
  virtual void onRetardo() = 0;
  virtual void onClock_1_00Sec() {};


  // --------------------------
  // Disparos
  // --------------------------
 public:
  void setDanio(int Danio);
  bool Desapareceme();
  void Dispara (oObjetos* Victima);
  oObjetos *oObjetos::getPresaOSeguir();
  bool IntentaDisparo();
  bool getArmaLista();

 private:
  virtual void onIntentaDisparo(bool Alcanzado){};
 private:
  virtual void onSetDanio(int Danio) = 0;


  // --------------------------
  // Sonidos
  // --------------------------
 public:
  virtual void PlaySeleccionado() = 0;
  virtual void PlayMuerto() = 0;
  virtual void PlayTerminado() = 0;
    
};

#endif
