//***************************************************************
//*  [[ A D V E R T E N C I A ]] =[ http://Solar-OpenSource.Org ]=
//*
//*  LICENCIA PARA LA DISTRIBUCIN DE WAR 3D
//*  1.- WAR 3D es un programa que podr ser distribuido bajo los trminos de la Licencia Pblica General, adems debern cumplirse las siguientes condiciones.
//*  2.- Al copiar cualquier parte del cdigo para ponerlo en un programa diferente a WAR 3D, el autor de dicho programa estar de acuerdo en que el cdigo completo de su programa pasa a ser software libre, por lo deber estar apegado a los trminos establecidos por la licencia LGP. Es decir que, el cdigo de WAR 3D solo puede ser tomado si se va a usar en otro proyecto de software libre. De lo contrario abstngase de usar el cdigo o parte de el cdigo de WAR3D.
//*  3.- Queda estrictamente prohibido modificar los nombres que aparecen en los crditos del programa, solo el grupo de desarrollo de  Solar-OpenSource podr aadir nuevos nombres en los crditos. Si ha hecho alguna modificacin y quiere que su nombre aparezca en los crditos debe ponerse en contacto con nosotros.
//*  4.- Cualquier modificacin del cdigo debe apegarse en lo posible a las normas que ha establecido el grupo de desarrollo de Solar-OpenSource, esto es para mayor claridad en el cdigo. Dichas normas pueden encontrarse en nuestra pagina web http://www.solar-opensource.org
//*  5.- Puede copiar y distribuir el programa siempre y cuando no olvide: Acompaarlo con el cdigo fuente completo correspondiente, legible para la mquina, as como el cdigo escrito por usted o las modificaciones hechas al cdigo original.
//*  6.- No debe distribuirse el ejecutable sin el cdigo fuente.
//*  7.- Este texto no debe suprimirse. Las distribuciones deben acompaarse con la informacin que se recibi as como el ofrecimiento para distribuir el cdigo fuente correspondiente.
//*
//***************************************************************

#ifndef __zeru_math__
#define __zeru_math__

#include <iostream>
#include <string>

using namespace std;

#include "constantes.h"

#include <math.h>

extern float __zcosArray[2160];
extern float __zsinArray[2160];
extern bool __zeruMatIni;




#define Pixel_H 0.0009765625  // 1024
#define Pixel_V 0.0013020833  // 768
//---------------------------------
// Vector 3D
typedef struct 
{
  float x, y, z;

}z3DVector;
//---------------------------------
// Vector 2D
typedef struct 
{
  float x, y;

}z2DVector;


void iniciaZeruMat (void);
float zcos(float t);
float zsin(float t);
int zsgn(float x);
float zangulo(float y, float x);
float zDistancia( float x1, float y1, float x2, float y2);


z3DVector CalcSuma(z3DVector Suma1, z3DVector Suma2);
z3DVector CalcNormalizar(z3DVector Normal);
z3DVector CalcCruz(z3DVector Vector1, z3DVector Vector2);
z3DVector CalcVector(z3DVector Punto1, z3DVector Punto2);
z3DVector CalcDividirPorEscalar(z3DVector Vector, float Escalar);

float GiroSuave(float Inicio, float Destino, int Retrazo);
z3DVector VectorSuave(z3DVector Inicio, z3DVector Destino, int Retrazo);
float LineaSuave (float Inicio, float Destino, int Retrazo);

/* __zeru_math__ */





/*
	Matrix.h

	Matrix manipulation.

	Author:	Brett Porter
	Email: brettporter@yahoo.com
	Website: http://rsn.gamedev.net/pl3d
	Copyright (C)2000, 2001, Brett Porter. All Rights Reserved.
	This source code is released under the LGPL. See license.txt for details.

	Created: 9 December 2000
	Last Edited: 8 January 2001

	Please see the file ChangeLog.html for a revision history.
*/



class Matrix
{
	public:
		/*	Constructor. */
		Matrix();

		/*	Destructor. */
		~Matrix();

		/*	Set to identity. */
		void loadIdentity();

		/*	Set the values of the matrix. */
		void set( const float *matrix );

		/*	Post-multiply by another matrix. */
		void postMultiply( const Matrix& matrix );

		/*	Set the translation of the current matrix. Will erase any previous values. */
		void setTranslation( const float *translation );

		/*	Set the inverse translation of the current matrix. Will erase any previous values. */
		void setInverseTranslation( const float *translation );

		/*	Make a rotation matrix from Euler angles. The 4th row and column are unmodified. */
		void setRotationRadians( const float *angles );

		/*	Make a rotation matrix from Euler angles. The 4th row and column are unmodified. */
		void setRotationDegrees( const float *angles );
#if 0
		/*	Make a rotation matrix from a quaternion. The 4th row and column are unmodified. */
		void setRotationQuaternion( const Quaternion& quat );
#endif
		/*	Make an inverted rotation matrix from Euler angles. The 4th row and column are unmodified. */
		void setInverseRotationRadians( const float *angles );

		/*	Make an inverted rotation matrix from Euler angles. The 4th row and column are unmodified. */
		void setInverseRotationDegrees( const float *angles );

		/*	Get the matrix data. */
		const float *getMatrix() const { return m_matrix; }

		void translateVect( float *pVect ) const;

		/*	Rotate a vector by the inverse of the rotation part of this matrix. */
		void rotateVect( float *pVect ) const;

		/*	Translate a vector by the inverse of the translation part of this matrix. */
		void inverseTranslateVect( float *pVect ) const;

		/*	Rotate a vector by the inverse of the rotation part of this matrix. */
		void inverseRotateVect( float *pVect ) const;

	private:
		//	Matrix data, stored in column-major order
		float m_matrix[16];
};

inline void Matrix::set( const float *matrix )
{
	memcpy( m_matrix, matrix, sizeof( float )*16 );
}

inline void Matrix::loadIdentity()
{
	memset( m_matrix, 0, sizeof( float )*16 );
	m_matrix[0] = m_matrix[5] = m_matrix[10] = m_matrix[15] = 1;
}

inline void Matrix::rotateVect( float *pVect ) const
{
	float vec[3];

	vec[0] = pVect[0]*m_matrix[0]+pVect[1]*m_matrix[4]+pVect[2]*m_matrix[8];
	vec[1] = pVect[0]*m_matrix[1]+pVect[1]*m_matrix[5]+pVect[2]*m_matrix[9];
	vec[2] = pVect[0]*m_matrix[2]+pVect[1]*m_matrix[6]+pVect[2]*m_matrix[10];

	memcpy( pVect, vec, sizeof( float )*3 );
}

inline void Matrix::inverseRotateVect( float *pVect ) const
{
	float vec[3];

	vec[0] = pVect[0]*m_matrix[0]+pVect[1]*m_matrix[1]+pVect[2]*m_matrix[2];
	vec[1] = pVect[0]*m_matrix[4]+pVect[1]*m_matrix[5]+pVect[2]*m_matrix[6];
	vec[2] = pVect[0]*m_matrix[8]+pVect[1]*m_matrix[9]+pVect[2]*m_matrix[10];

	memcpy( pVect, vec, sizeof( float )*3 );
}

inline void Matrix::translateVect( float *pVect ) const
{
	pVect[0] = pVect[0]+m_matrix[12];
	pVect[1] = pVect[1]+m_matrix[13];
	pVect[2] = pVect[2]+m_matrix[14];
}

inline void Matrix::inverseTranslateVect( float *pVect ) const
{
	pVect[0] = pVect[0]-m_matrix[12];
	pVect[1] = pVect[1]-m_matrix[13];
	pVect[2] = pVect[2]-m_matrix[14];
}






#endif                  






